#include </usr/include/unistd.h> // ugly hack to get the right unistd.h (doxygen has one too)
#include "doxygen.h"

static void listSymbols()
{
  SDict<DefinitionList>::Iterator sli(*Doxygen::symbolMap);
  DefinitionList *dl;
  // for each list of symbols with the same name
  for (sli.toFirst();(dl=sli.current());++sli)
  {
    DefinitionListIterator dli(*dl);
    Definition *d;
    // for each symbol
    for (dli.toFirst();(d=dli.current());++dli)
    {
      if (d!=Doxygen::globalScope && // skip the global namespace symbol
          d->name().at(0)!='@'       // skip anonymous stuff
         )      
      {
        printf("%s\n",
            d->name().data());
      }
    }
  }
}

static void lookupSymbol(const QCString &sym)
{
  if (!sym.isEmpty())
  {
    DefinitionList *dl = Doxygen::symbolMap->find(sym);
    if (dl)
    {
      DefinitionListIterator dli(*dl);
      Definition *d;
      // for each symbol with the given name
      for (dli.toFirst();(d=dli.current());++dli)
      {
        if (d!=Doxygen::globalScope && // skip the global namespace symbol
            d->name().at(0)!='@'       // skip anonymous stuff
           )      
        {
          printf("Symbol info\n");
          printf("-----------\n");
          printf("Name: %s\n",d->name().data());
          printf("File: %s\n",d->getDefFileName().data());
          printf("Line: %d\n",d->getDefLine());
          // depending on the definition type we can case to the appropriate
          // derived to get additional information
          switch (d->definitionType())
          {
            case Definition::TypeClass:
              {
                ClassDef *cd = (ClassDef *)d;
                printf("Kind: %s\n",cd->compoundTypeString().data());
              }
              break;
            case Definition::TypeFile:
              {
                FileDef *fd = (FileDef *)d;
                printf("Kind: File: #includes %d other files\n",
                    fd->includeFileList() ? fd->includeFileList()->count() : 0);
              }
              break;
            case Definition::TypeNamespace:
              {
                NamespaceDef *nd = (NamespaceDef *)d;
                printf("Kind: Namespace: contains %d classes and %d namespaces\n",
                    nd->classSDict     ? nd->classSDict->count() : 0,
                    nd->namespaceSDict ? nd->namespaceSDict->count() : 0);
              }
              break;
            case Definition::TypeMember:
              {
                MemberDef *md = (MemberDef *)d;
                printf("Kind: %s\n",md->memberTypeName().data());
              }
              break;
            default:
              // ignore groups/pages/packages/dirs for now
              break;
          }
        }
      }
    }
    else
    {
      printf("Unknown symbol\n");
    }
  }
}

int main(int argc,char **argv)
{
  char cmd[256];

  if (argc<2)
  {
    printf("Usage: %s [source_file | source_dir]\n",argv[0]);
    exit(1);
  }

  // initialize data structures 
  initDoxygen();

  // setup the non-default configuration options

  // we need a place to put intermediate files
  Config_getString("OUTPUT_DIRECTORY")="/tmp/doxygen"; 
  // disable html output
  Config_getBool("GENERATE_HTML")=FALSE;
  // disable latex output
  Config_getBool("GENERATE_LATEX")=FALSE;
  // be quiet
  Config_getBool("QUIET")=TRUE;
  // turn off warnings
  Config_getBool("WARNINGS")=FALSE;
  Config_getBool("WARN_IF_UNDOCUMENTED")=FALSE;
  Config_getBool("WARN_IF_DOC_ERROR")=FALSE;
  // set the input
  Config_getList("INPUT").append(argv[1]);

  // check and finialize the configuration
  checkConfiguration();

  // parse the file
  parseInput();

  // clean up after us
  rmdir("/tmp/doxygen");

  while (1)
  {
    printf("> Type a symbol name or @ for a list of symbols (Ctrl-C to exit)\n> ");
    fgets(cmd,256,stdin);
    if (cmd[0]=='@')
    {
      listSymbols();
    }
    else
    {
      QCString s(cmd);
      if (s.at(s.length()-1)=='\n') s=s.left(s.length()-1); // strip trailing \n
      lookupSymbol(s);
    }
  }
}

